@@ -1,5 +1,9 @@
-import time, warnings, sys, os.path as osp
+import warnings, os.path as osp
+
+from ..dss_plugin_mlflow import MLflowHandle
+
 from .dataset import DSSDataset, DSSDatasetListItem, DSSManagedDatasetCreationHelper
+from .modelcomparison import DSSModelComparison
 from .jupyternotebook import DSSJupyterNotebook, DSSJupyterNotebookListItem
 from .notebook import DSSNotebook
 from .streaming_endpoint import DSSStreamingEndpoint, DSSStreamingEndpointListItem, DSSManagedStreamingEndpointCreationHelper
@@ -8,6 +12,7 @@ from . import recipe
 from .managedfolder import DSSManagedFolder
 from .savedmodel import DSSSavedModel
 from .modelevaluationstore import DSSModelEvaluationStore
+from .mlflow import DSSMLflowExtension
 from .job import DSSJob, DSSJobWaiter
 from .scenario import DSSScenario, DSSScenarioListItem
 from .continuousactivity import DSSContinuousActivity
@@ -16,21 +21,21 @@ from .future import DSSFuture
 from .macro import DSSMacro
 from .wiki import DSSWiki
 from .discussion import DSSObjectDiscussions
-from .ml import DSSMLTask
+from .ml import DSSMLTask, DSSMLTaskQueues
 from .analysis import DSSAnalysis
 from .flow import DSSProjectFlow
 from .app import DSSAppManifest
-
+from .codestudio import DSSCodeStudioObject, DSSCodeStudioObjectListItem
 
 class DSSProject(object):
     """
     A handle to interact with a project on the DSS instance.
 
-    Do not create this class directly, instead use :meth:`dataikuapi.DSSClient.get_project``
+    Do not create this class directly, instead use :meth:`dataikuapi.DSSClient.get_project`
     """
     def __init__(self, client, project_key):
-       self.client = client
-       self.project_key = project_key
+        self.client = client
+        self.project_key = project_key
 
     def get_summary(self):
         """
@@ -69,7 +74,7 @@ class DSSProject(object):
     def move_to_folder(self, folder):
         """
         Moves this project to a project folder
-        :param folder :class:`dataikuapi.dss.projectfolder.DSSProjectFolder
+        :param folder :class:`dataikuapi.dss.projectfolder.DSSProjectFolder`
         """
         current_folder = self.get_project_folder()
         current_folder.move_project_to(self.project_key, folder)
@@ -78,17 +83,25 @@ class DSSProject(object):
     # Project deletion
     ########################################################
 
-    def delete(self, drop_data=False):
+    def delete(self, clear_managed_datasets=False, clear_output_managed_folders=False, clear_job_and_scenario_logs=True, **kwargs):
         """
         Delete the project
 
         This call requires an API key with admin rights
 
-        :param bool drop_data: Should the data of managed datasets be dropped
+        :param bool clear_managed_datasets: Should the data of managed datasets be cleared
+        :param bool clear_output_managed_folders: Should the data of managed folders used as outputs of recipes be cleared
+        :param bool clear_job_and_scenario_logs: Should the job and scenario logs be cleared
         """
+        # For backwards compatibility
+        if 'drop_data' in kwargs and kwargs['drop_data']:
+            clear_managed_datasets = True
+
         return self.client._perform_empty(
-            "DELETE", "/projects/%s" % self.project_key, params = {
-                "dropData": drop_data
+            "DELETE", "/projects/%s" % self.project_key, params={
+                "clearManagedDatasets": clear_managed_datasets,
+                "clearOutputManagedFolders": clear_output_managed_folders,
+                "clearJobAndScenarioLogs": clear_job_and_scenario_logs
             })
 
     ########################################################
@@ -132,11 +145,12 @@ class DSSProject(object):
             * exportManagedFS (boolean): Exports the data of managed Filesystem datasets - default False
             * exportAnalysisModels (boolean): Exports the models trained in analysis - default False
             * exportSavedModels (boolean): Exports the models trained in saved models - default False
+            * exportModelEvaluationStores (boolean): Exports the evaluation stores - default False
             * exportManagedFolders (boolean): Exports the data of managed folders - default False
             * exportAllInputDatasets (boolean): Exports the data of all input datasets - default False
             * exportAllDatasets (boolean): Exports the data of all datasets - default False
             * exportAllInputManagedFolders (boolean): Exports the data of all input managed folders - default False
-            * exportGitRepositoy (boolean): Exports the Git repository history - default False
+            * exportGitRepository (boolean): Exports the Git repository history - default False
             * exportInsightsData (boolean): Exports the data of static insights - default False
         """
         if options is None:
@@ -273,7 +287,7 @@ class DSSProject(object):
           - createdBy: who created this project
           - createdOn: when the project was created
           - lastModifiedBy: who modified this project for the last time
-          - lastModifiedBy: when this modification took place
+          - lastModifiedOn: when this modification took place
         :rtype: dict
         """
         return self.client._perform_json("GET", "/projects/%s/timeline" % self.project_key, params = {
@@ -638,6 +652,15 @@ class DSSProject(object):
         """
         return DSSMLTask(self.client, self.project_key, analysis_id, mltask_id)
 
+    def list_mltask_queues(self):
+        """
+        List non-empty ML task queues in this project 
+        
+        :returns: an iterable :class:`DSSMLTaskQueues` listing of MLTask queues (each a dict)
+        :rtype: :class:`DSSMLTaskQueues`
+        """ 
+        data = self.client._perform_json("GET", "/projects/%s/models/labs/mltask-queues" % self.project_key)
+        return DSSMLTaskQueues(data)
 
     def create_analysis(self, input_dataset):
         """
@@ -699,6 +722,22 @@ class DSSProject(object):
         """
         return DSSSavedModel(self.client, self.project_key, sm_id)
 
+    def create_mlflow_pyfunc_model(self, name, prediction_type = None):
+        """
+        Creates a new external saved model for storing and managing MLFlow models
+
+        :param string name: Human readable name for the new saved model in the flow
+        :param string prediction_type: Optional (but needed for most operations). One of BINARY_CLASSIFICATION, MULTICLASS or REGRESSION
+        """
+        model = {
+            "savedModelType" : "MLFLOW_PYFUNC",
+            "predictionType" : prediction_type,
+            "name": name
+        }
+
+        id = self.client._perform_json("POST", "/projects/%s/savedmodels/" % self.project_key, body = model)["id"]
+        return self.get_saved_model(id)
+
     ########################################################
     # Managed folders
     ########################################################
@@ -754,18 +793,15 @@ class DSSProject(object):
     # Model evaluation stores
     ########################################################
 
-    def list_model_evaluation_stores(self, as_type=None):
+    def list_model_evaluation_stores(self):
         """
         List the model evaluation stores in this project.
 
         :returns: The list of the model evaluation stores
-        :rtype: list
+        :rtype: list of :class:`dataikuapi.dss.modelevaluationstore.DSSModelEvaluationStore`
         """
         items = self.client._perform_json("GET", "/projects/%s/modelevaluationstores/" % self.project_key)
-        if as_type == "objects" or as_type == "object":
-            return [DSSModelEvaluationStore(self.client, self.project_key, item["id"]) for item in items]
-        else:
-            return items
+        return [DSSModelEvaluationStore(self.client, self.project_key, item["id"]) for item in items]
 
     def get_model_evaluation_store(self, mes_id):
         """
@@ -777,17 +813,15 @@ class DSSProject(object):
         """
         return DSSModelEvaluationStore(self.client, self.project_key, mes_id)
 
-    def create_model_evaluation_store(self, name, mes_id=None):
+    def create_model_evaluation_store(self, name):
         """
         Create a new model evaluation store in the project, and return a handle to interact with it.
         
         :param string name: the name for the new model evaluation store
-        :param string mes_id optional: the id for the new model evaluation store
-        
+
         :returns: A :class:`dataikuapi.dss.modelevaluationstore.DSSModelEvaluationStore` model evaluation store handle
         """
         obj = {
-            "id" : mes_id,
             "projectKey" : self.project_key,
             "name" : name
         }
@@ -796,6 +830,51 @@ class DSSProject(object):
         mes_id = res['id']
         return DSSModelEvaluationStore(self.client, self.project_key, mes_id)
 
+    ########################################################
+    # Model comparisons
+    ########################################################
+
+    def list_model_comparisons(self):
+        """
+        List the model comparisons in this project.
+
+        :returns: The list of the model comparisons
+        :rtype: list
+        """
+        items = self.client._perform_json("GET", "/projects/%s/modelcomparisons/" % self.project_key)
+        return [DSSModelComparison(self.client, self.project_key, item["id"]) for item in items]
+
+    def get_model_comparison(self, mec_id):
+        """
+        Get a handle to interact with a specific model comparison
+
+        :param string mec_id: the id of the desired model comparison
+
+        :returns: A handle on a model comparison
+        :rtype: :class:`dataikuapi.dss.modelcomparison.DSSModelComparison`
+        """
+        return DSSModelComparison(self.client, self.project_key, mec_id)
+
+    def create_model_comparison(self, name, prediction_type):
+        """
+        Create a new model comparison in the project, and return a handle to interact with it.
+
+        :param string name: the name for the new model comparison
+        :param string prediction_type: one of BINARY_CLASSIFICATION, REGRESSION and MULTICLASS
+
+        :returns: A handle on a new model comparison
+        :rtype: :class:`dataikuapi.dss.modelcomparison.DSSModelComparison`
+        """
+        obj = {
+            "projectKey": self.project_key,
+            "displayName": name,
+            "predictionType": prediction_type
+        }
+        res = self.client._perform_json("POST", "/projects/%s/modelcomparisons/" % self.project_key,
+                                        body = obj)
+        mec_id = res['id']
+        return DSSModelComparison(self.client, self.project_key, mec_id)
+
     ########################################################
     # Jobs
     ########################################################
@@ -823,14 +902,13 @@ class DSSProject(object):
         """
         Create a new job, and return a handle to interact with it
         
-        Args:
-            definition: the definition for the job to create. The definition must contain the type of job (RECURSIVE_BUILD, 
-            NON_RECURSIVE_FORCED_BUILD, RECURSIVE_FORCED_BUILD, RECURSIVE_MISSING_ONLY_BUILD) and a list of outputs to build.
-            Optionally, a refreshHiveMetastore field can specify whether to re-synchronize the Hive metastore for recomputed
-            HDFS datasets.
+        :param dict definition: The definition should contain: 
+            
+            * the type of job (RECURSIVE_BUILD, NON_RECURSIVE_FORCED_BUILD, RECURSIVE_FORCED_BUILD, RECURSIVE_MISSING_ONLY_BUILD)
+            * a list of outputs to build from the available types: (DATASET, MANAGED_FOLDER, SAVED_MODEL, STREAMING_ENDPOINT)
+            * (Optional) a refreshHiveMetastore field (True or False) to specify whether to re-synchronize the Hive metastore for recomputed HDFS datasets.
         
-        Returns:
-            A :class:`dataikuapi.dss.job.DSSJob` job handle
+        :returns: A :class:`dataikuapi.dss.job.DSSJob` job handle
         """
         job_def = self.client._perform_json("POST", "/projects/%s/jobs/" % self.project_key, body = definition)
         return DSSJob(self.client, self.project_key, job_def['id'])
@@ -839,11 +917,11 @@ class DSSProject(object):
         """
         Starts a new job and waits for it to complete.
         
-        Args:
-            definition: the definition for the job to create. The definition must contain the type of job (RECURSIVE_BUILD, 
-            NON_RECURSIVE_FORCED_BUILD, RECURSIVE_FORCED_BUILD, RECURSIVE_MISSING_ONLY_BUILD) and a list of outputs to build.
-            Optionally, a refreshHiveMetastore field can specify whether to re-synchronize the Hive metastore for recomputed
-            HDFS datasets.
+        :param dict definition: The definition should contain:
+            
+            * the type of job (RECURSIVE_BUILD, NON_RECURSIVE_FORCED_BUILD, RECURSIVE_FORCED_BUILD, RECURSIVE_MISSING_ONLY_BUILD)
+            * a list of outputs to build from the available types: (DATASET, MANAGED_FOLDER, SAVED_MODEL, STREAMING_ENDPOINT)
+            * (Optional) a refreshHiveMetastore field (True or False) to specify whether to re-synchronize the Hive metastore for recomputed HDFS datasets.
         """
         job_def = self.client._perform_json("POST", "/projects/%s/jobs/" % self.project_key, body = definition)
         job = DSSJob(self.client, self.project_key, job_def['id'])
@@ -883,8 +961,8 @@ class DSSProject(object):
         :param bool as_type: How to return the list. Supported values are "listitems" and "objects".
         :param bool active: if True, only return currently running jupyter notebooks.
 
-        :returns: The list of the notebooks. If "as_type" is "listitems", each one as a :class:`dataikuapi.dss.notebook.DSSJupyterNotebookListItem`,
-        if "as_type" is "objects", each one as a :class:`dataikuapi.dss.notebook.DSSJupyterNotebook`
+        :returns: The list of the notebooks. If "as_type" is "listitems", each one as a :class:`dataikuapi.dss.notebook.DSSJupyterNotebookListItem`, if "as_type" is "objects", each one as a :class:`dataikuapi.dss.notebook.DSSJupyterNotebook`
+        
         :rtype: list of :class:`dataikuapi.dss.notebook.DSSJupyterNotebook` or list of :class:`dataikuapi.dss.notebook.DSSJupyterNotebookListItem`
         """
         notebook_items = self.client._perform_json("GET", "/projects/%s/jupyter-notebooks/" % self.project_key, params={"active": active})
@@ -968,7 +1046,7 @@ class DSSProject(object):
         WARNING: if executed from a python recipe, the changes made by `set_variables` will not be "seen" in that recipe.
                  Use the internal API dataiku.get_custom_variables() instead if this behavior is needed
 
-        @param dict obj: must be a modified version of the object returned by get_variables
+        :param dict obj: must be a modified version of the object returned by get_variables
         """
         if not "standard" in obj:
             raise ValueError("Missing 'standard' key in argument")
@@ -1035,17 +1113,39 @@ class DSSProject(object):
     ########################################################
 
     def list_exported_bundles(self):
+        """
+        :returns: A dictionary of all bundles for a project on the Design node.
+        """
         return self.client._perform_json("GET",
                 "/projects/%s/bundles/exported" % self.project_key)
 
     def export_bundle(self, bundle_id):
+        """
+        Creates a new project bundle on the Design node 
+        
+        :param str bundle_id: bundle id tag 
+        """
         return self.client._perform_json("PUT",
                 "/projects/%s/bundles/exported/%s" % (self.project_key, bundle_id))
 
     def get_exported_bundle_archive_stream(self, bundle_id):
         """
         Download a bundle archive that can be deployed in a DSS automation Node, as a binary stream.
-        Warning: this stream will monopolize the DSSClient until closed.
+        
+        .. warning::
+            
+            The stream must be closed after use. Use a ``with`` statement to handle closing the stream at the end of the block by default. For example:
+            
+            .. code-block:: python 
+        
+                    with project.get_exported_bundle_archive_stream('v1') as fp:
+                        # use fp 
+                    
+                    # or explicitly close the stream after use
+                    fp = project.get_exported_bundle_archive_stream('v1')
+                    # use fp, then close 
+                    fp.close()
+                    
         """
         return self.client._perform_raw("GET",
                 "/projects/%s/bundles/exported/%s/archive" % (self.project_key, bundle_id))
@@ -1054,7 +1154,7 @@ class DSSProject(object):
         """
         Download a bundle archive that can be deployed in a DSS automation Node into the given output file.
         
-        :param path if "-", will write to /dev/stdout
+        :param string path: if "-", will write to /dev/stdout
         """
         if path == "-":
             path= "/dev/stdout"
@@ -1090,15 +1190,34 @@ class DSSProject(object):
     ########################################################
 
     def list_imported_bundles(self):
+        """
+        :returns: a dict containing bundle imports for a project, on the Automation node.
+        """
         return self.client._perform_json("GET",
                 "/projects/%s/bundles/imported" % self.project_key)
 
     def import_bundle_from_archive(self, archive_path):
+        """
+        Imports a bundle from a zip archive path on the Automation node.
+        
+        :param str archive_path: A full path to a zip archive, for example `/home/dataiku/my-bundle-v1.zip`
+        """
         return self.client._perform_json("POST",
                 "/projects/%s/bundles/imported/actions/importFromArchive" % (self.project_key),
                  params = { "archivePath" : osp.abspath(archive_path) })
 
     def import_bundle_from_stream(self, fp):
+        """
+        Imports a bundle from a file stream, on the Automation node. 
+        
+        :param file-like fp: file handler. Usage example: 
+        
+        .. code-block:: python
+
+            project = client.get_project('MY_PROJECT')
+            with open('/home/dataiku/my-bundle-v1.zip', 'rb') as f:
+                project.import_bundle_from_stream(f)
+        """
         files = {'file': fp}
         return self.client._perform_empty("POST",
                 "/projects/%s/bundles/imported/actions/importFromStream" % (self.project_key),
@@ -1120,6 +1239,11 @@ class DSSProject(object):
                 "/projects/%s/bundles/imported/%s/actions/activate" % (self.project_key, bundle_id), body=options)
 
     def preload_bundle(self, bundle_id):
+        """
+        Preloads a bundle that has been imported on the Automation node 
+        
+        :param str bundle_id: the bundle_id for an existing imported bundle
+        """
         return self.client._perform_json("POST",
                 "/projects/%s/bundles/imported/%s/actions/preload" % (self.project_key, bundle_id))
 
@@ -1200,8 +1324,9 @@ class DSSProject(object):
     def get_recipe(self, recipe_name):
         """
         Gets a :class:`dataikuapi.dss.recipe.DSSRecipe` handle to interact with a recipe
+        
         :param str recipe_name: The name of the recipe
-        :rtype :class:`dataikuapi.dss.recipe.DSSRecipe`
+        :rtype: :class:`dataikuapi.dss.recipe.DSSRecipe`
         """
         return DSSRecipe(self.client, self.project_key, recipe_name)
 
@@ -1281,6 +1406,10 @@ class DSSProject(object):
             return recipe.PrepareRecipeCreator(name, self)
         elif type == "prediction_scoring":
             return recipe.PredictionScoringRecipeCreator(name, self)
+        elif type == "evaluation":
+            return recipe.EvaluationRecipeCreator(name, self)
+        elif type == "standalone_evaluation":
+            return recipe.StandaloneEvaluationRecipeCreator(name, self)
         elif type == "clustering_scoring":
             return recipe.ClusteringScoringRecipeCreator(name, self)
         elif type == "download":
@@ -1295,6 +1424,9 @@ class DSSProject(object):
     ########################################################
 
     def get_flow(self):
+        """
+        :rtype: A :class:`dataikuapi.dss.flow.DSSProjectFlow`
+        """
         return DSSProjectFlow(self.client, self)
 
     ########################################################
@@ -1461,7 +1593,7 @@ class DSSProject(object):
         """
         connection_name = "@virtual(hive-jdbc):" + hive_database
         ret = self.client._perform_json("GET", "/projects/%s/datasets/tables-import/actions/list-tables" % (self.project_key),
-                params = {"connectionName": connection_name} )
+                params={"connectionName": connection_name} )
 
         def to_schema_table_pair(x):
             return {"schema":x.get("databaseName", None), "table":x["table"]}
@@ -1470,11 +1602,81 @@ class DSSProject(object):
     ########################################################
     # App designer
     ########################################################
-
     def get_app_manifest(self):
         raw_data = self.client._perform_json("GET", "/projects/%s/app-manifest" % self.project_key)
         return DSSAppManifest(self.client, raw_data, self.project_key)
 
+    ########################################################
+    # MLflow experiment tracking
+    ########################################################
+    def setup_mlflow(self, managed_folder, host=None):
+        """
+        Setup the dss-plugin for MLflow
+
+        :param object managed_folder: the managed folder where MLflow artifacts should be stored.
+                                      Can be either a managed folder id as a string,
+                                      a :class:`dataikuapi.dss.DSSManagedFolder`, or a :class:`dataiku.Folder`
+        :param str host: setup a custom host if the backend used is not DSS.
+        """
+        return MLflowHandle(client=self.client, project=self, managed_folder=managed_folder, host=host)
+
+    def get_mlflow_extension(self):
+        """
+        Get a handle to interact with the extension of MLflow provided by DSS
+
+        :returns: A :class:`dataikuapi.dss.mlflow.DSSMLflowExtension` Mlflow Extension handle
+
+        """
+        return DSSMLflowExtension(client=self.client, project_key=self.project_key)
+
+
+
+    ########################################################
+    # Code studios
+    ########################################################
+    def list_code_studios(self, as_type="listitems"):
+        """
+        List the code studio objects in this project
+        
+        :returns: the list of the code studio objects, each one as a JSON object
+        """
+        items = self.client._perform_json(
+            "GET", "/projects/%s/code-studios/" % self.project_key)
+        if as_type == "listitems" or as_type == "listitem":
+            return [DSSCodeStudioObjectListItem(self.client, self.project_key, item) for item in items]
+        elif as_type == "objects" or as_type == "object":
+            return [DSSCodeStudioObject(self.client, self.project_key, item["id"]) for item in items]
+        else:
+            raise ValueError("Unknown as_type") 
+
+    def get_code_studio(self, code_studio_id):
+        """
+        Get a handle to interact with a specific code studio object
+       
+        :param str code_studio_id: the identifier of the desired code studio object
+        
+        :returns: A :class:`dataikuapi.dss.codestudio.DSSCodeStudioObject` code studio object handle
+        """
+        return DSSCodeStudioObject(self.client, self.project_key, code_studio_id)
+
+    def create_code_studio(self, name, template_id):
+        """
+        Create a new code studio object in the project, and return a handle to interact with it
+        
+        :param str name: the name of the code studio object
+        :param str template_id: the identifier of a code studio template
+        
+        :returns: A :class:`dataikuapi.dss.codestudio.DSSCodeStudioObject` code studio object handle
+        """
+        obj = {
+            "name" : name,
+            "templateId" : template_id
+        }
+        res = self.client._perform_json("POST", "/projects/%s/code-studios/" % self.project_key, body = obj)
+        code_studio_id = res['codeStudio']['id']
+        return DSSCodeStudioObject(self.client, self.project_key, code_studio_id)
+
+
 
 class TablesImportDefinition(object):
     """
@@ -1681,6 +1883,11 @@ class JobDefinitionBuilder(object):
     def with_output(self, name, object_type=None, object_project_key=None, partition=None):
         """
         Adds an item to build in this job
+        
+        :param name: name of the output object
+        :param object_type: type of object to build from: DATASET, MANAGED_FOLDER, SAVED_MODEL, STREAMING_ENDPOINT
+        :param object_project_key: PROJECT_KEY for the project that contains the object to build 
+        :param partition: specify partition to build
         """
         self.definition['outputs'].append({'type':object_type, 'id':name, 'projectKey':object_project_key, 'partition':partition})
         return self
