@@ -3,6 +3,10 @@ from .utils import DSSTaggableObjectSettings
 from .discussion import DSSObjectDiscussions
 import json, logging, warnings
 from .utils import DSSTaggableObjectListItem, DSSTaggableObjectSettings
+try:
+    basestring
+except NameError:
+    basestring = str
 
 class DSSRecipeListItem(DSSTaggableObjectListItem):
     """An item in a list of recipes. Do not instantiate this class, use :meth:`dataikuapi.dss.project.DSSProject.list_recipes`"""
@@ -34,6 +38,11 @@ class DSSRecipe(object):
         self.project_key = project_key
         self.recipe_name = recipe_name
 
+    @property
+    def id(self):
+        """The id of the recipe"""
+        return self.recipe_name
+
     @property
     def name(self):
         """The name of the recipe"""
@@ -89,6 +98,7 @@ class DSSRecipe(object):
             "COMPUTABLE_FOLDER": "MANAGED_FOLDER",
             "COMPUTABLE_SAVED_MODEL": "SAVED_MODEL",
             "COMPUTABLE_STREAMING_ENDPOINT": "STREAMING_ENDPOINT",
+            "COMPUTABLE_MODEL_EVALUATION_STORE": "MODEL_EVALUATION_STORE"
         }
         if first_output["type"] in object_type_map:
             jd = project.new_job(job_type)
@@ -97,7 +107,7 @@ class DSSRecipe(object):
             raise Exception("Recipe has unsupported output type {}, can't run it".format(first_output["type"]))
 
         if wait:
-            return jd.start_and_wait()
+            return jd.start_and_wait(no_fail)
         else:
             return jd.start()
 
@@ -201,7 +211,7 @@ class DSSRecipe(object):
     def set_metadata(self, metadata):
         """
         Set the metadata on this recipe.
-        :params dict metadata: the new state of the metadata for the recipe. You should only set a metadata object 
+        :params dict metadata: the new state of the metadata for the recipe. You should only set a metadata object
             that has been retrieved using the get_metadata call.
         """
         return self.client._perform_json(
@@ -224,8 +234,18 @@ class DSSRecipe(object):
         from .continuousactivity import DSSContinuousActivity
         return DSSContinuousActivity(self.client, self.project_key, self.recipe_name)
 
+    def move_to_zone(self, zone):
+        """
+        Moves this object to a flow zone
+
+        :param object zone: a :class:`dataikuapi.dss.flow.DSSFlowZone` where to move the object
+        """
+        if isinstance(zone, basestring):
+           zone = self.client.get_project(self.project_key).get_flow().get_zone(zone)
+        zone.add_item(self)
+
 class DSSRecipeStatus(object):
-    """Status of a recipce. 
+    """Status of a recipce.
     Do not create that directly, use :meth:`DSSRecipe.get_status`"""
 
     def __init__(self, client, data):
@@ -269,7 +289,7 @@ class DSSRecipeStatus(object):
         """
         Returns status messages for this recipe.
 
-        :returns: a list of dict, for each status message. Each dict represents a single message, 
+        :returns: a list of dict, for each status message. Each dict represents a single message,
             and contains at least a "severity" field (SUCCESS, WARNING or ERROR)
             and a "message" field
         :rtype: list
@@ -612,7 +632,7 @@ class DSSRecipeCreator(object):
 
     def create(self):
         """
-        Creates the new recipe in the project, and return a handle to interact with it. 
+        Creates the new recipe in the project, and return a handle to interact with it.
 
         Returns:
             A :class:`dataikuapi.dss.recipe.DSSRecipe` recipe handle
@@ -787,7 +807,7 @@ class GroupingRecipeCreator(SingleOutputRecipeCreator):
 
     def with_group_key(self, group_key):
         """
-        Set a column as the first grouping key. Only a single grouping key may be set 
+        Set a column as the first grouping key. Only a single grouping key may be set
         at recipe creation time. For additional groupings, get the recipe settings
 
         :param str group_key: name of a column in the input dataset
@@ -1030,6 +1050,13 @@ class FuzzyJoinRecipeCreator(VirtualInputsSingleOutputRecipeCreator):
     def __init__(self, name, project):
         VirtualInputsSingleOutputRecipeCreator.__init__(self, 'fuzzyjoin', name, project)
 
+class GeoJoinRecipeCreator(VirtualInputsSingleOutputRecipeCreator):
+    """
+    Create a GeoJoin recipe
+    """
+    def __init__(self, name, project):
+        VirtualInputsSingleOutputRecipeCreator.__init__(self, 'geojoin', name, project)
+
 class StackRecipeSettings(DSSRecipeSettings):
     """
     Settings of a stack recipe. Do not create this directly, use :meth:`DSSRecipe.get_settings`
@@ -1175,8 +1202,8 @@ class CodeRecipeCreator(DSSRecipeCreator):
         :param str connection_id: name of the connection to create the dataset on
         :param str type: type of dataset, for connection where the type could be ambiguous. Typically,
                                  this is SCP or SFTP, for SSH connection
-        :param str format: name of a format preset relevant for the dataset type. Possible values are: CSV_ESCAPING_NOGZIP_FORHIVE, 
-                                     CSV_UNIX_GZIP, CSV_EXCEL_GZIP, CSV_EXCEL_GZIP_BIGQUERY, CSV_NOQUOTING_NOGZIP_FORPIG, PARQUET_HIVE, 
+        :param str format: name of a format preset relevant for the dataset type. Possible values are: CSV_ESCAPING_NOGZIP_FORHIVE,
+                                     CSV_UNIX_GZIP, CSV_EXCEL_GZIP, CSV_EXCEL_GZIP_BIGQUERY, CSV_NOQUOTING_NOGZIP_FORPIG, PARQUET_HIVE,
                                      AVRO, ORC. If None, uses the default
         :param str copy_partitioning_from: Whether to copy the partitioning from another thing.
                     Use None for not partitioning the output, "FIRST_INPUT" to copy from the first input of the recipe,
@@ -1313,6 +1340,161 @@ class PredictionScoringRecipeCreator(SingleOutputRecipeCreator):
         return self._with_input(model_id, self.project.project_key, "model")
 
 
+class EvaluationRecipeCreator(DSSRecipeCreator):
+    """
+    Builder for the creation of a new "Evaluate" recipe, from an
+    input dataset, with an input saved model identifier
+
+    .. code-block:: python
+
+        # Create a new evaluation recipe outputing to a new dataset, to a metrics dataset and/or to a model evaluation store
+
+        project = client.get_project("MYPROJECT")
+        builder = project.new_recipe("evaluation")
+        builder.with_input_model(saved_model_id)
+        builder.with_input("dataset_to_evaluate")
+
+        builder.with_output("output_scored")
+        builder.with_output_metrics("output_metrics")
+        builder.with_output_evaluation_store(evaluation_store_id)
+
+        new_recipe = builder.build()
+
+        # Access the settings
+
+        er_settings = new_recipe.get_settings()
+        payload = er_settings.obj_payload
+
+        # Change the settings
+
+        payload['dontComputePerformance'] = True
+        payload['outputProbabilities'] = False
+        payload['metrics'] = ["precision", "recall", "auc", "f1", "costMatrixGain"]
+
+        # Manage evaluation labels
+
+        payload['labels'] = [dict(key="label_1", value="value_1"), dict(key="label_2", value="value_2")]
+
+        # Save the settings and run the recipe
+
+        er_settings.save()
+
+        new_recipe.run()
+
+    Outputs must exist. They can be created using the following:
+
+    .. code-block:: python
+
+        builder = project.new_managed_dataset("output_scored")
+        builder.with_store_into(connection)
+        dataset = builder.create()
+
+        builder = project.new_managed_dataset("output_scored")
+        builder.with_store_into(connection)
+        dataset = builder.create()
+
+        evaluation_store_id = project.create_model_evaluation_store("output_model_evaluation").mes_id
+    """
+
+    def __init__(self, name, project):
+        DSSRecipeCreator.__init__(self, 'evaluation', name, project)
+
+    def with_input_model(self, model_id):
+        """Sets the input model"""
+        return self._with_input(model_id, self.project.project_key, "model")
+
+    def with_output(self, name):
+        """Sets the ouput dataset containing the scored input"""
+        return self._with_output(name, role="main")
+
+    def with_output_metrics(self, name):
+        """Sets the output dataset containing the metrics"""
+        return self._with_output(name, role="metrics")
+
+    def with_output_evaluation_store(self, mes_id):
+        """Sets the output model evaluation store"""
+        return self._with_output(mes_id, role="evaluationStore")
+
+
+class StandaloneEvaluationRecipeCreator(DSSRecipeCreator):
+    """
+    Builder for the creation of a new "Standalone Evaluate" recipe, from an
+    input dataset
+
+    .. code-block:: python
+
+        # Create a new standalone evaluation of a scored dataset
+
+        project = client.get_project("MYPROJECT")
+        builder = project.new_recipe("standalone_evaluation")
+        builder.with_input("scored_dataset_to_evaluate")
+        builder.with_output_evaluation_store(evaluation_store_id)
+
+        # Add a reference dataset (optional) to compute data drift
+
+        builder.with_reference_dataset("reference_dataset")
+
+        # Finish creation of the recipe
+
+        new_recipe = builder.create()
+
+        # Modify the model parameters in the SER settings
+
+        ser_settings = new_recipe.get_settings()
+        payload = ser_settings.obj_payload
+
+        payload['predictionType'] = "BINARY_CLASSIFICATION"
+        payload['targetVariable'] = "Survived"
+        payload['predictionVariable'] = "prediction"
+        payload['isProbaAware'] = True
+        payload['dontComputePerformance'] = False
+
+        # For a classification model with probabilities, the 'probas' section can be filled with the mapping of the class and the probability column
+        # e.g. for a binary classification model with 2 columns: proba_0 and proba_1
+
+        class_0 = dict(key=0, value="proba_0")
+        class_1 = dict(key=1, value="proba_1")
+        payload['probas'] = [class_0, class_1]
+
+        # Change the 'features' settings for this standalone evaluation
+        # e.g. reject the features that you do not want to use in the evaluation
+
+        feature_passengerid = dict(name="Passenger_Id", role="REJECT", type="TEXT")
+        feature_ticket = dict(name="Ticket", role="REJECT", type="TEXT")
+        feature_cabin = dict(name="Cabin", role="REJECT", type="TEXT")
+
+        payload['features'] = [feature_passengerid, feature_ticket, feature_cabin]
+
+        # To set the cost matrix properly, access the 'metricParams' section of the payload and set the cost matrix weights:
+
+        payload['metricParams'] = dict(costMatrixWeights=dict(tpGain=0.4, fpGain=-1.0, tnGain=0.2, fnGain=-0.5))
+
+        # Save the recipe and run the recipe
+        # Note that with this method, all the settings that were not explicitly set are instead set to their default value.
+
+        ser_settings.save()
+
+        new_recipe.run()
+
+    Output model evaluation store must exist. It can be created using the following:
+
+    .. code-block:: python
+
+        evaluation_store_id = project.create_model_evaluation_store("output_model_evaluation").mes_id
+    """
+
+    def __init__(self, name, project):
+        DSSRecipeCreator.__init__(self, 'standalone_evaluation', name, project)
+
+    def with_output_evaluation_store(self, mes_id):
+        """Sets the output model evaluation store"""
+        return self._with_output(mes_id, role="main")
+
+    def with_reference_dataset(self, dataset_name):
+        """Sets the dataset to use as a reference in data drift computation (optional)."""
+        return self._with_input(dataset_name, self.project.project_key, role="reference")
+
+
 class ClusteringScoringRecipeCreator(SingleOutputRecipeCreator):
     """
     Builder for the creation of a new "Clustering scoring" recipe, from an
@@ -1343,44 +1525,3 @@ class ClusteringScoringRecipeCreator(SingleOutputRecipeCreator):
     def with_input_model(self, model_id):
         """Sets the input model"""
         return self._with_input(model_id, self.project.project_key, "model")
-
-
-class DownloadRecipeCreator(SingleOutputRecipeCreator):
-    """
-    Create a Download recipe
-    """
-    def __init__(self, name, project):
-        SingleOutputRecipeCreator.__init__(self, 'download', name, project)
-
-
-class RequiredSchemaUpdates(object):
-    """
-    Representation of the updates required to the schema of the outputs of a recipe.
-    Do not create this class directly, use :meth:`DSSRecipe.compute_schema_updates`
-    """
-
-    def __init__(self, recipe, data):
-        self.recipe = recipe
-        self.data = data
-        self.drop_and_recreate = True
-        self.synchronize_metastore = True
-
-    def any_action_required(self):
-        return self.data["totalIncompatibilities"] > 0
-
-    def apply(self):
-        results  = []
-        for computable in self.data["computables"]:
-            osu = {
-                "computableType": computable["type"],
-                # dirty
-                "computableId": computable["type"] == "DATASET" and computable["datasetName"] or computable["id"],
-                "newSchema": computable["newSchema"],
-                "dropAndRecreate": self.drop_and_recreate,
-                "synchronizeMetastore" : self.synchronize_metastore
-            }
-
-            results.append(self.recipe.client._perform_json("POST",
-                    "/projects/%s/recipes/%s/actions/updateOutputSchema" % (self.recipe.project_key, self.recipe.recipe_name),
-                    body=osu))
-        return results
